#!/usr/bin/python
# coding:utf-8

import boto3
from boto3.session import Session
import botocore

import sys
import time

class EMR_Helper:
    ################################################################################
    # 定数群
    ################################################################################


    # retry間隔
    INTERVAL = 1.0

    # コンストラクタ
    def __init__(self, credential):
        session = Session(aws_access_key_id     = credential['accessKey'],
                          aws_secret_access_key = credential['secretKey'],
                          region_name           = credential['region'])

        self.__client = session.client('emr')


    ################################################################################
    # Throttling対策関数群
    ################################################################################

    # list_clusters
    def __list_clusters_force_1st(self, cluster_states):
        try:
            clusters = self.__client.list_clusters(
                ClusterStates = cluster_states
            )
        except botocore.exceptions.ClientError:
            if not 'ThrottlingException' in sys.exc_info():
                raise
            time.sleep(EMR_Helper.INTERVAL)
            clusters = self.__list_clusters_force_1st(cluster_states)

        return clusters

    def __list_clusters_force(self, cluster_states, next_token):
        try:
            clusters = self.__client.list_clusters(
                ClusterStates = cluster_states,
                Marker        = next_token,
            )
        except botocore.exceptions.ClientError:
            if not 'ThrottlingException' in sys.exc_info():
                raise
            time.sleep(EMR_Helper.INTERVAL)
            clusters = self.__list_clusters_force(cluster_states, next_token)

        return clusters

    # describe_cluster
    def describe_cluster_force(self, cluster_id):
        try:
            cluster_info = self.__client.describe_cluster(
                ClusterId  = cluster_id,
            )
        except:
            if not 'ThrottlingException' in sys.exc_info():
                raise
            time.sleep(EMR_Helper.INTERVAL)
            cluster_info = self.describe_cluster_force(cluster_id)

        return cluster_info

    # list_steps
    def list_steps_force(self, cluster_id, states):
        try:
            steps = self.__client.list_steps(
                ClusterId  = cluster_id,
                StepStates = states
            )
        except botocore.exceptions.ClientError:
            if not 'ThrottlingException' in sys.exc_info():
                raise
            time.sleep(EMR_Helper.INTERVAL)
            steps = self.list_steps_force(cluster_id, states)

        return steps


    ################################################################################
    # ヘルパー関数群
    ################################################################################

    # 各リソースに特定の処理をする
    def exec_func_each_clusters(self, states_filter, func, **args):
        first_call = True
        result     = True
        while True:
            if first_call:
                resources = self.__list_clusters_force_1st(states_filter)
                first_call = False
            else:
                resources = self.__list_clusters_force(states_filter, next_token)

            for cluster in resources['Clusters']:
                ret = func(cluster, args)
                if not ret:
                    result = False

            # 次のページなし
            if not resources.has_key('Marker'):
                break
            # 次のページへのポインタを取得
            next_token = resources['Marker']

        return result



#!/usr/bin/python
# coding:utf-8

import boto3
from boto3.session import Session
import botocore

import sys
import time

class EC2_Helper:
    ################################################################################
    # 定数群
    ################################################################################

    # コア数定義
    TYPE_CORE_1 = {
        't2.nano', 't2.micro', 't2.small',
        'm3.medium'
    }
    TYPE_CORE_2 = {
        't2.medium', 't2.large',
        'm4.large',
        'm3.large',
        'c4.large',
        'c3.large',
        'r3.large',
    }
    TYPE_CORE_4 = {
        'm4.xlarge',
        'm3.xlarge',
        'c4.xlarge',
        'c3.xlarge',
        'r3.xlarge',
        'i2.xlarge',
        'd2.xlarge',
    }
    TYPE_CORE_8 = {
        'm4.2xlarge',
        'm3.2xlarge',
        'c4.2xlarge',
        'c3.2xlarge',
        'g2.2xlarge',
        'r3.2xlarge',
        'i2.2xlarge',
        'd2.2xlarge',
    }
    TYPE_CORE_16 = {
        'm4.4xlarge',
        'c4.4xlarge',
        'c3.4xlarge',
        'r3.4xlarge',
        'i2.4xlarge',
        'd2.4xlarge',
    }
    TYPE_CORE_32 = {
        'c3.8xlarge',
        'g2.8xlarge',
        'r3.8xlarge',
        'i2.8xlarge',
        'd2.8xlarge',
    }
    TYPE_CORE_36 = {
        'c4.8xlarge',
    }
    TYPE_CORE_40 = {
        'm4.10xlarge',
    }

    # retry間隔
    INTERVAL = 1.0

    # コンストラクタ
    def __init__(self, credential=None):
        if credential is None:
            self.__client = boto3.client('ec2')

        else:
            session = Session(aws_access_key_id     = credential['accessKey'],
                              aws_secret_access_key = credential['secretKey'],
                              region_name           = credential['region'])

            self.__client = session.client('ec2')

    

    ################################################################################
    # Throttling対策関数群
    ################################################################################
    def __describe_instances_force_1st(self):
        try:
            response = self.__client.describe_instances()
        except botocore.exceptions.ClientError:
            if not 'ThrottlingException' in sys.exc_info():
                raise
            time.sleep(EC2_Helper.INTERVAL)
            response = self.__describe_instances_force_1st()

        return response

    def __describe_instances_force(self, next_token):
        try:
            response = self.__client.describe_instances(
                NextToken = next_token
            )
        except botocore.exceptions.ClientError:
            if not 'ThrottlingException' in sys.exc_info():
                raise
            time.sleep(EC2_Helper.INTERVAL)
            response = self.__describe_instances_force(next_token)

        return response


    ################################################################################
    # ヘルパー関数群
    ################################################################################

    # 各リソースに特定の処理をする
    def exec_func_each_instances(self, func, **args):
        first_call = True
        result     = True
        while True:
            if first_call:
                resources = self.__describe_instances_force_1st()
                first_call = False
            else:
                resources = self.__describe_instances_force(next_token)

            for reservation in resources['Reservations']:
                instances = reservation['Instances']
                for instance in instances:
                    ret = func(instance, args)
                    if not ret:
                        result = False

            # 次のページなし
            if not resources.has_key('NextToken'):
                break
            # 次のページへのポインタを取得
            next_token = resources['NextToken']

        return result


    # インスタンスタイプからコア数を判定
    @staticmethod
    def judge_core_count(instance_type):
        if   instance_type in EC2_Helper.TYPE_CORE_1:
            return 1
        elif instance_type in EC2_Helper.TYPE_CORE_2:
            return 2
        elif instance_type in EC2_Helper.TYPE_CORE_4:
            return 4
        elif instance_type in EC2_Helper.TYPE_CORE_8:
            return 8
        elif instance_type in EC2_Helper.TYPE_CORE_16:
            return 16
        elif instance_type in EC2_Helper.TYPE_CORE_32:
            return 32
        elif instance_type in EC2_Helper.TYPE_CORE_36:
            return 36
        elif instance_type in EC2_Helper.TYPE_CORE_40:
            return 40

        return 0



#!/usr/bin/python
# coding:utf-8

import boto3
from boto3.session import Session
import botocore

import sys
import time

class CloudWatchLogs_Helper:
    ################################################################################
    # 定数群
    ################################################################################


    # retry間隔
    INTERVAL = 1.0

    # コンストラクタ
    def __init__(self, credential=None):
        if credential is None:
            self.__client = boto3.client('logs')

        else:
            session = Session(aws_access_key_id     = credential['accessKey'],
                              aws_secret_access_key = credential['secretKey'],
                              region_name           = credential['region'])

            self.__client = session.client('logs')


    ################################################################################
    # Throttling対策関数群
    ################################################################################

    # describe_log_streams
    def __describe_log_streams_force_1st(self, group_name):
        try:
            streams = self.__client.describe_log_streams(
                logGroupName = group_name,
                descending   = True,
                orderBy      = 'LastEventTime',
            )
        except botocore.exceptions.ClientError:
            if not 'ThrottlingException' in sys.exc_info():
                raise
            time.sleep(CloudWatchLogs_Helper.INTERVAL)
            streams = self.__describe_log_streams_force(group_name)

        return streams

    def __describe_log_streams_force(self, group_name, next_token):
        try:
            streams = self.__client.describe_log_streams(
                logGroupName = group_name,
                descending   = True,
                orderBy      = 'LastEventTime',
                nextToken     = next_token,
            )
        except botocore.exceptions.ClientError:
            if not 'ThrottlingException' in sys.exc_info():
                raise
            time.sleep(CloudWatchLogs_Helper.INTERVAL)
            streams = self.__describe_log_streams_force(group_name, next_token)

        return streams

    # get_log_events
    def __get_log_events_force_1st(self, group_name, stream_name):
        try:
            log_events = self.__client.get_log_events(
                logGroupName  = group_name,
                logStreamName = stream_name,
                startFromHead = True
            )
        except botocore.exceptions.ClientError:
            if not 'ThrottlingException' in sys.exc_info():
                raise
            time.sleep(CloudWatchLogs_Helper.INTERVAL)
            log_events = self.__get_log_events_force_1st(group_name, stream_name)

        return log_events

    def __get_log_events_force(self, group_name, stream_name, next_token):
        try:
            log_events = self.__client.get_log_events(
                logGroupName  = group_name,
                logStreamName = stream_name,
                nextToken     = next_token,
                startFromHead = True
            )
        except botocore.exceptions.ClientError:
            if not 'ThrottlingException' in sys.exc_info():
                raise
            time.sleep(CloudWatchLogs_Helper.INTERVAL)
            log_events = self.__get_log_events_force(group_name, stream_name, next_token)

        return log_events


    ################################################################################
    # ヘルパー関数群
    ################################################################################

    # 各ストリームに特定の処理をする
    def exec_func_each_streams(self, group_name, func, **args):
        first_call = True
        while True:
            if first_call:
                streams = self.__describe_log_streams_force_1st(group_name)
                first_call = False
            else:
                streams = self.__describe_log_streams_force(group_name, next_token)

            for stream in streams['logStreams']:
                ret = func(stream, args)
                if not ret:
                    result = False

            # 次のページなし
            if not streams.has_key('nextToken'):
                break
            # 次のページへのポインタを取得
            next_token = streams['nextToken']

        return result

    # 各イベントに特定の処理をする
    def exec_func_each_events(self, group_name, stream_name, func, **args):
        first_call = True
        while True:
            if first_call:
                log_events = self.__get_log_events_force_1st(group_name, stream_name)
                first_call = False
            else:
                log_events = self.__get_log_events_force(group_name, stream_name, next_token)

            events = log_events['events']
            events_cnt = len(events)
            if events_cnt == 0:
                # CloudWatchLogsは取得イベント数がゼロの際にページャー末端となる
                break

            events.reverse()
            for event in events:
                time.sleep(0.0001)  # CPU張り付き対策

                ret = func(event, args)
                if not ret:
                    result = False

            # 次のページへのポインタを取得
            next_token = log_events['nextForwardToken']

        return result



import re
import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)

def read(*parts):
    # intentionally *not* adding an encoding option to open
    # see here: https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    return open(path.join(here, *parts), 'r').read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='boto3helper',
    version=find_version('boto3helper', '__init__.py'),
    description='boto3 helper classes',
    long_description=long_description,
    url='https://github.com/Hiroyama-Yutaka/boto3helper.git',
    author='Yutaka Hiroyama',
    author_email='hiroyama@mbk.nifty.com',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
    ],
    keywords=['AWS',],
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    install_requires=['boto3', 'botocore'],
 )
